import {CLIOption} from "./CLIOption"

/**
 * Non-positional, flag-based CLI option.
 */
export class FlagOption<T> extends CLIOption<T> {

    constructor(
        public readonly longFlag?: string,
        public readonly shortFlag?: string,
        public readonly message?: string,
        public readonly argumentDescription?: string
    ) { super() }

    /**
     * Get the referential name for this option.
     * Defaults to the long flag (without the '--'). If this cannot
     * be found, the short flag (without the '-') is used.
     * @returns {string}
     */
    getArgumentName() {
        if ( this.longFlag ) {
            return this.longFlag.replace('--', '')
        } else if ( this.shortFlag ) {
            return this.shortFlag.replace('-', '')
        }

        throw new Error('Missing either a long- or short-flag for FlagOption.')
    }
}
